@@ -13,8 +13,6 @@
 #include <string.h>
 #include "../util.h"
 
-#include "openbsd-compat.h"
-
 static void test_nouserok(const char *username) {
   device_t *dev;
   unsigned ndevs;
@@ -749,7 +747,7 @@ static void test_new_credentials(const char *username) {
 }
 
 int main(void) {
-  struct passwd *pwd;
+  const struct passwd *pwd;
   char *username;
 
   assert((pwd = getpwuid(geteuid())) != NULL);
